@@ -8,11 +8,17 @@
   The Abyss component of Xmlrpc-c is based on the independently developed
   and distributed Abyss web server package from 2001.
 
+  Nothing may include this header file that also includes <winsock.h>,
+  because it conflicts with this file's use of <winsock2.h>.  Furthermore,
+  nothing including this file may include <windows.h> without previously
+  defining WIN32_LEAN_AND_MEAN, because <windows.h> without that macro
+  includes <winsock.h> automatically.  (see abyss_winsock.h).
+
   Copyright information is at the end of the file.
 ****************************************************************************/
 
-#ifndef _ABYSS_H_
-#define _ABYSS_H_
+#ifndef XMLRPC_ABYSS_H_INCLUDED
+#define XMLRPC_ABYSS_H_INCLUDED
 
 
 #ifdef __cplusplus
@@ -21,10 +27,20 @@ extern "C" {
 
 #include <sys/types.h>
 
-#ifdef WIN32
-#include "xmlrpc_config.h"
+#include <xmlrpc-c/c_util.h>  /* For XMLRPC_DLLEXPORT */
+#include <xmlrpc-c/inttypes.h>
+
+/*
+  XMLRPC_ABYSS_EXPORTED marks a symbol in this file that is exported from
+  libxmlrpc_abyss.
+
+  XMLRPC_BUILDING_ABYSS says this compilation is part of libxmlrpc_abyss, as
+  opposed to something that _uses_ libxmlrpc_abyss.
+*/
+#ifdef XMLRPC_BUILDING_ABYSS
+#define XMLRPC_ABYSS_EXPORTED XMLRPC_DLLEXPORT
 #else
-#include <inttypes.h>
+#define XMLRPC_ABYSS_EXPORTED
 #endif
 
 /****************************************************************************
@@ -33,29 +49,47 @@ extern "C" {
 
 typedef int abyss_bool;
 
+/****************************************************************************
+  GLOBAL (STATIC) PROGRAM STUFF
+****************************************************************************/
+
+XMLRPC_ABYSS_EXPORTED
+void
+AbyssInit(const char ** const errorP);
+
+XMLRPC_ABYSS_EXPORTED
+void
+AbyssTerm(void);
+
 /*********************************************************************
 ** MIMEType
 *********************************************************************/
 
 typedef struct MIMEType MIMEType;
 
+XMLRPC_ABYSS_EXPORTED
 MIMEType *
 MIMETypeCreate(void);
 
+XMLRPC_ABYSS_EXPORTED
 void
 MIMETypeDestroy(MIMEType * const MIMETypeP);
 
+XMLRPC_ABYSS_EXPORTED
 void
 MIMETypeInit(void);
 
+XMLRPC_ABYSS_EXPORTED
 void
 MIMETypeTerm(void);
 
+XMLRPC_ABYSS_EXPORTED
 abyss_bool
 MIMETypeAdd2(MIMEType *   const MIMETypeP,
              const char * const type,
              const char * const ext);
 
+XMLRPC_ABYSS_EXPORTED
 abyss_bool
 MIMETypeAdd(const char * const type,
             const char * const ext);
@@ -63,35 +97,48 @@ MIMETypeAdd(const char * const type,
 
 enum abyss_foreback {ABYSS_FOREGROUND, ABYSS_BACKGROUND};
 
+#define HAVE_CHANSWITCH
 
+typedef struct _TChanSwitch TChanSwitch;
+typedef struct _TChannel TChannel;
 typedef struct _TSocket TSocket;
 
-/* TOsSocket is the type of a conventional socket offered by our OS.
-   This is for backward compatibility; everyone should use TSocket
-   sockets today.
+#ifdef _WIN32
+  #include <xmlrpc-c/abyss_winsock.h>
+#else
+  #include <xmlrpc-c/abyss_unixsock.h>
+#endif
+
+XMLRPC_ABYSS_EXPORTED
+void
+ChanSwitchInit(const char ** const errorP);
+
+XMLRPC_ABYSS_EXPORTED
+void
+ChanSwitchTerm(void);
+
+/* If you're wondering where the constructors for TChanSwitch,
+   TChannel, and TSocket are: They're implementation-specific, so look
+   in abyss_unixsock.h, etc.
 */
-#ifdef WIN32
+
+XMLRPC_ABYSS_EXPORTED
 void
-SocketWinCreate(TSocket ** const socketPP);
+ChanSwitchDestroy(TChanSwitch * const chanSwitchP);
 
+XMLRPC_ABYSS_EXPORTED
 void
-SocketWinCreateWinsock(SOCKET     const winsock,
-                       TSocket ** const socketPP);
-
-typedef SOCKET TOsSocket;
-
-#else  /* WIN32 */
+ChannelInit(const char ** const errorP);
 
+XMLRPC_ABYSS_EXPORTED
 void
-SocketUnixCreate(TSocket ** const socketPP);
+ChannelTerm(void);
 
+XMLRPC_ABYSS_EXPORTED
 void
-SocketUnixCreateFd(int        const fd,
-                   TSocket ** const socketPP);
-
-typedef int TOsSocket;
-#endif  /* WIN32 */
+ChannelDestroy(TChannel * const channelP);
 
+XMLRPC_ABYSS_EXPORTED
 void
 SocketDestroy(TSocket * const socketP);
 
@@ -108,13 +155,21 @@ typedef struct {
 
 typedef struct _TSession TSession;
 
+XMLRPC_ABYSS_EXPORTED
 abyss_bool
-ServerCreate(TServer *    const serverP,
-             const char * const name,
-             uint16_t     const port,
-             const char * const filespath,
-             const char * const logfilename);
+ServerCreate(TServer *       const serverP,
+             const char *    const name,
+             xmlrpc_uint16_t const port,
+             const char *    const filespath,
+             const char *    const logfilename);
 
+XMLRPC_ABYSS_EXPORTED
+void
+ServerCreateSwitch(TServer *     const serverP,
+                   TChanSwitch * const chanSwitchP,
+                   const char ** const errorP);
+
+XMLRPC_ABYSS_EXPORTED
 abyss_bool
 ServerCreateSocket(TServer *    const serverP,
                    const char * const name,
@@ -123,94 +178,139 @@ ServerCreateSocket(TServer *    const serverP,
                    const char * const logfilename);
 
 #define HAVE_SERVER_CREATE_SOCKET_2
+XMLRPC_ABYSS_EXPORTED
 void
 ServerCreateSocket2(TServer *     const serverP,
                     TSocket *     const socketP,
                     const char ** const errorP);
 
+XMLRPC_ABYSS_EXPORTED
 abyss_bool
 ServerCreateNoAccept(TServer *    const serverP,
                      const char * const name,
                      const char * const filespath,
                      const char * const logfilename);
 
+XMLRPC_ABYSS_EXPORTED
 void
 ServerFree(TServer * const serverP);
 
+XMLRPC_ABYSS_EXPORTED
 void
 ServerSetName(TServer *    const serverP,
               const char * const name);
 
+XMLRPC_ABYSS_EXPORTED
 void
 ServerSetFilesPath(TServer *    const serverP,
                    const char * const filesPath);
 
+XMLRPC_ABYSS_EXPORTED
 void
 ServerSetLogFileName(TServer *    const serverP,
                      const char * const logFileName);
 
 #define HAVE_SERVER_SET_KEEPALIVE_TIMEOUT 1
+XMLRPC_ABYSS_EXPORTED
 void
-ServerSetKeepaliveTimeout(TServer * const serverP,
-                          uint32_t  const keepaliveTimeout);
+ServerSetKeepaliveTimeout(TServer *       const serverP,
+                          xmlrpc_uint32_t const keepaliveTimeout);
 
 #define HAVE_SERVER_SET_KEEPALIVE_MAX_CONN 1
+XMLRPC_ABYSS_EXPORTED
 void
-ServerSetKeepaliveMaxConn(TServer * const serverP,
-                          uint32_t  const keepaliveMaxConn);
+ServerSetKeepaliveMaxConn(TServer *       const serverP,
+                          xmlrpc_uint32_t const keepaliveMaxConn);
 
 #define HAVE_SERVER_SET_TIMEOUT 1
+XMLRPC_ABYSS_EXPORTED
 void
-ServerSetTimeout(TServer * const serverP,
-                 uint32_t  const timeout);
+ServerSetTimeout(TServer *       const serverP,
+                 xmlrpc_uint32_t const timeout);
 
 #define HAVE_SERVER_SET_ADVERTISE 1
+XMLRPC_ABYSS_EXPORTED
 void
 ServerSetAdvertise(TServer *  const serverP,
                    abyss_bool const advertise);
 
 #define HAVE_SERVER_SET_MIME_TYPE 1
+XMLRPC_ABYSS_EXPORTED
 void
 ServerSetMimeType(TServer *  const serverP,
                   MIMEType * const MIMETypeP);
 
+#define HAVE_SERVER_SET_MAX_CONN 1
+XMLRPC_ABYSS_EXPORTED
+void
+ServerSetMaxConn(TServer *    const serverP,
+                 unsigned int const maxConn);
+
+#define HAVE_SERVER_SET_MAX_CONN_BACKLOG 1
+XMLRPC_ABYSS_EXPORTED
+void
+ServerSetMaxConnBacklog(TServer *    const serverP,
+                        unsigned int const maxConnBacklog);
+
+XMLRPC_ABYSS_EXPORTED
+void
+ServerInit2(TServer *     const serverP,
+            const char ** const errorP);
+
+XMLRPC_ABYSS_EXPORTED
 void
 ServerInit(TServer * const serverP);
 
+XMLRPC_ABYSS_EXPORTED
 void
 ServerRun(TServer * const serverP);
 
+XMLRPC_ABYSS_EXPORTED
 void
 ServerRunOnce(TServer * const serverP);
 
 /* ServerRunOnce2() is obsolete.  See user's guide. */
+XMLRPC_ABYSS_EXPORTED
 void
 ServerRunOnce2(TServer *           const serverP,
                enum abyss_foreback const foregroundBackground);
 
+XMLRPC_ABYSS_EXPORTED
 void
-ServerRunConn(TServer * const serverP,
-              TOsSocket const connectedSocket);
+ServerRunChannel(TServer *     const serverP,
+                 TChannel *    const channelP,
+                 void *        const channelInfoP,
+                 const char ** const errorP);
 
 #define HAVE_SERVER_RUN_CONN_2
+XMLRPC_ABYSS_EXPORTED
 void
 ServerRunConn2(TServer *     const serverP,
                TSocket *     const connectedSocketP,
                const char ** const errorP);
 
+XMLRPC_ABYSS_EXPORTED
+void
+ServerRunConn(TServer * const serverP,
+              TOsSocket const connectedSocket);
+
+XMLRPC_ABYSS_EXPORTED
 void
 ServerDaemonize(TServer * const serverP);
 
+XMLRPC_ABYSS_EXPORTED
 void
 ServerTerminate(TServer * const serverP);
 
+XMLRPC_ABYSS_EXPORTED
 void
 ServerResetTerminate(TServer * const serverP);
 
+XMLRPC_ABYSS_EXPORTED
 void
 ServerUseSigchld(TServer * const serverP);
 
-#ifndef WIN32
+#ifndef _WIN32
 void
 ServerHandleSigchld(pid_t const pid);
 #endif
@@ -219,15 +319,31 @@ typedef abyss_bool (*URIHandler) (TSession *); /* deprecated */
 
 struct URIHandler2;
 
-typedef void (*initHandlerFn)(struct URIHandler2 *,
-                              abyss_bool *);
+typedef void (*initHandlerFn)(struct URIHandler2 *, abyss_bool *);
 
 typedef void (*termHandlerFn)(void *);
 
+typedef void (*handleReq3Fn)(void *,
+                             TSession *,
+                             abyss_bool *);
+
 typedef void (*handleReq2Fn)(struct URIHandler2 *,
                              TSession *,
                              abyss_bool *);
 
+struct ServerReqHandler3 {
+    termHandlerFn term;
+    handleReq3Fn  handleReq;
+    void *        userdata;
+    size_t        handleReqStackSize; /* zero = default */
+};
+
+XMLRPC_ABYSS_EXPORTED
+void
+ServerAddHandler3(TServer *                        const serverP,
+                  const struct ServerReqHandler3 * const handlerP,
+                  abyss_bool *                     const successP);
+
 typedef struct URIHandler2 {
     initHandlerFn init;
     termHandlerFn term;
@@ -236,33 +352,45 @@ typedef struct URIHandler2 {
     void *        userdata;
 } URIHandler2;
 
+XMLRPC_ABYSS_EXPORTED
 void
 ServerAddHandler2(TServer *     const srvP,
                   URIHandler2 * const handlerP,
                   abyss_bool *  const successP);
 
+XMLRPC_ABYSS_EXPORTED
 abyss_bool
 ServerAddHandler(TServer * const srvP,
                  URIHandler const handler);
 
-void
-ServerDefaultHandler(TServer *  const srvP,
-                     URIHandler const handler);
+typedef abyss_bool (*THandlerDflt) (TSession *);
 
-/* This is inappropriately named; it was a mistake.  But then, so is
-   having this function at all.  The config file is inappropriate for
-   an API.
+/* Note: 'handler' used to be URIHandler;  THandlerDflt is a newer name
+   for the same type
 */
+
+XMLRPC_ABYSS_EXPORTED
+void
+ServerDefaultHandler(TServer *    const srvP,
+                     THandlerDflt const handler);
+
+/* ConfReadServerFile() is inappropriately named; it was a mistake.
+   But then, so is having this function at all.  The config file is
+   inappropriate for an API.
+*/
+
+XMLRPC_ABYSS_EXPORTED
 abyss_bool
 ConfReadServerFile(const char * const filename,
                    TServer *    const srvP);
 
+XMLRPC_ABYSS_EXPORTED
 void
 LogWrite(TServer *    const srvP,
          const char * const c);
 
 /****************************************************************************
-  STUFF FOR URI HANDLERS TO USE
+  STUFF FOR HTTP REQUEST HANDLERS TO USE
 ****************************************************************************/
 
 typedef enum {
@@ -274,104 +402,191 @@ typedef struct {
     const char * uri;
         /* This is NOT the URI.  It is the pathname part of the URI.
            We really should fix that and put the pathname in another
-           member.
+           member.  If the URI does not contain a pathname, this is "*".
         */
     const char * query;
-        /* The query part of the URI (stuff after '?') */
+        /* The query part of the URI (stuff after '?').  NULL if none. */
     const char * host;
-        /* NOT the value of the host: header.  Rather, the name of the
-           target host (could be part of the host: value).  No port number.
+        /* NOT the value of the host: header field.  Rather, the name of the
+           target host (could be part of the host: value; could be from the
+           URI).  No port number.  NULL if request does not specify a host
+           name.
         */
     const char * from;
     const char * useragent;
     const char * referer;
     const char * requestline;
     const char * user;
-    unsigned short port;
+        /* Requesting user (from authorization: header field).  NULL if
+           request doesn't specify or handler has not authenticated it.
+        */
+    xmlrpc_uint16_t port;
+        /* The port number from the URI, or default 80 if the URI doesn't
+           specify a port.
+        */
     abyss_bool keepalive;
 } TRequestInfo;
 
+XMLRPC_ABYSS_EXPORTED
 abyss_bool
 SessionRefillBuffer(TSession * const sessionP);
 
+XMLRPC_ABYSS_EXPORTED
 size_t
 SessionReadDataAvail(TSession * const sessionP);
 
+XMLRPC_ABYSS_EXPORTED
 void
 SessionGetReadData(TSession *    const sessionP, 
                    size_t        const max, 
                    const char ** const outStartP, 
                    size_t *      const outLenP);
 
+XMLRPC_ABYSS_EXPORTED
 void
 SessionGetRequestInfo(TSession *            const sessionP,
                       const TRequestInfo ** const requestInfoPP);
 
-char *
-RequestHeaderValue(TSession * const sessionP,
-                   char *     const name);
+XMLRPC_ABYSS_EXPORTED
+void
+SessionGetChannelInfo(TSession * const sessionP,
+                      void **    const channelInfoPP);
 
+XMLRPC_ABYSS_EXPORTED
+void *
+SessionGetDefaultHandlerCtx(TSession * const sessionP);
+
+XMLRPC_ABYSS_EXPORTED
+char *
+RequestHeaderValue(TSession *   const sessionP,
+                   const char * const name);
+
+XMLRPC_ABYSS_EXPORTED
+abyss_bool
+RequestAuth(TSession *   const sessionP,
+            const char * const credential,
+            const char * const user,
+            const char * const pass);
+
+XMLRPC_ABYSS_EXPORTED
 abyss_bool
 ResponseAddField(TSession *   const sessionP,
                  const char * const name,
                  const char * const value);
 
+XMLRPC_ABYSS_EXPORTED
 void
 ResponseWriteStart(TSession * const sessionP);
 
 /* For backward compatibility: */
 #define ResponseWrite ResponseWriteStart
 
+XMLRPC_ABYSS_EXPORTED
 abyss_bool
-ResponseWriteBody(TSession *   const sessionP,
-                  const char * const data,
-                  uint32_t     const len);
+ResponseWriteBody(TSession *      const sessionP,
+                  const char *    const data,
+                  xmlrpc_uint32_t const len);
 
+XMLRPC_ABYSS_EXPORTED
 abyss_bool
 ResponseWriteEnd(TSession * const sessionP);
 
+XMLRPC_ABYSS_EXPORTED
 abyss_bool
 ResponseChunked(TSession * const sessionP);
 
-uint16_t
+XMLRPC_ABYSS_EXPORTED
+xmlrpc_uint16_t
 ResponseStatusFromErrno(int const errnoArg);
 
+XMLRPC_ABYSS_EXPORTED
 void
-ResponseStatus(TSession * const sessionP,
-               uint16_t   const code);
+ResponseStatus(TSession *      const sessionP,
+               xmlrpc_uint16_t const code);
 
+XMLRPC_ABYSS_EXPORTED
 void
 ResponseStatusErrno(TSession * const sessionP);
 
+XMLRPC_ABYSS_EXPORTED
 abyss_bool
 ResponseContentType(TSession *   const serverP,
                     const char * const type);
 
+XMLRPC_ABYSS_EXPORTED
 abyss_bool
-ResponseContentLength(TSession * const sessionP,
-                      uint64_t   const len);
+ResponseContentLength(TSession *      const sessionP,
+                      xmlrpc_uint64_t const len);
 
+typedef struct {
+/*----------------------------------------------------------------------------
+   These are parameters to control the HTTP "Access Control" functions.  That's
+   where the client asks whether it is OK to send a request that some other
+   server asked the client to send (e.g. a person web browses a page at
+   a.example.com, and it sends a script that executes on the user's computer
+   and tries to perform an XML-RPC RPC on b.example.com.  The user's browser
+   first asks b.example.com if it is OK to do an RPC that is really initiated
+   by a.example.com.
+
+   These parameters tell the server how to respond to such a request.
+-----------------------------------------------------------------------------*/
+    const char * allowOrigin;
+        /* This tells what original servers (a.example.com in the example
+           above) are allowed to submit RPCs indirectly to us.  The value is a
+           verbatim value for an HTTP Access-Control-Allow-Origin header field
+           (just the value part of the field, not the whole field).  "*"
+           therefore means everyone is allowed.  "" means no one.
+    
+           NULL means not to say anything about access control to the client.
+        */
+    abyss_bool expires;
+        /* The permissions herein expire after a certain period from now.
+           'maxAge' is that period.
+        */
+    unsigned int maxAge;
+        /* Meaningful only when 'expires' is true.  The expiration period
+           in seconds.  Zero is valid.
+        */
+} ResponseAccessCtl;
+
+XMLRPC_ABYSS_EXPORTED
+void
+ResponseAccessControl(TSession *        const abyssSessionP, 
+                      ResponseAccessCtl const accessControl);
+
+XMLRPC_ABYSS_EXPORTED
+void
+ResponseError2(TSession *   const sessionP,
+               const char * const explanation);
+
+XMLRPC_ABYSS_EXPORTED
 void
 ResponseError(TSession * const sessionP);
 
+XMLRPC_ABYSS_EXPORTED
 const char *
 MIMETypeFromExt(const char * const ext);
 
+XMLRPC_ABYSS_EXPORTED
 const char *
 MIMETypeFromExt2(MIMEType *   const MIMETypeP,
                  const char * const ext);
 
+XMLRPC_ABYSS_EXPORTED
 const char *
 MIMETypeFromFileName2(MIMEType *   const MIMETypeP,
                       const char * const fileName);
 
+XMLRPC_ABYSS_EXPORTED
 const char *
 MIMETypeFromFileName(const char * const fileName);
 
+XMLRPC_ABYSS_EXPORTED
 const char *
 MIMETypeGuessFromFile2(MIMEType *   const MIMETypeP,
                        const char * const fileName);
 
+XMLRPC_ABYSS_EXPORTED
 const char *
 MIMETypeGuessFromFile(const char * const filename);
 
@@ -383,15 +598,11 @@ MIMETypeGuessFromFile(const char * const filename);
 ****************************************************************************/
 
 
-#define CR      '\r'
-#define LF      '\n'
-#define CRLF    "\r\n"
-
 /*********************************************************************
 ** Paths and so on...
 *********************************************************************/
 
-#ifdef WIN32
+#ifdef _WIN32
 #define DEFAULT_ROOT        "c:\\abyss"
 #define DEFAULT_DOCS        DEFAULT_ROOT"\\htdocs"
 #define DEFAULT_CONF_FILE   DEFAULT_ROOT"\\conf\\abyss.conf"
@@ -407,84 +618,26 @@ MIMETypeGuessFromFile(const char * const filename);
 #define DEFAULT_LOG_FILE    DEFAULT_ROOT"/log/abyss.log"
 #endif
 
-/*********************************************************************
-** Maximum number of simultaneous connections
-*********************************************************************/
-
-#define MAX_CONN    16
-
-/*********************************************************************
-** General purpose definitions
-*********************************************************************/
-
-#ifndef NULL
-#define NULL ((void *)0)
-#endif  /* NULL */
-
-#ifndef TRUE
-#define TRUE    1
-#endif  /* TRUE */
-
-#ifndef FALSE
-#define FALSE    0
-#endif  /* FALSE */
-
-/*********************************************************************
-** Buffer
-*********************************************************************/
-
-typedef struct
-{
-    void *data;
-    uint32_t size;
-    uint32_t staticid;
-} TBuffer;
-
-abyss_bool BufferAlloc(TBuffer *buf,uint32_t memsize);
-abyss_bool BufferRealloc(TBuffer *buf,uint32_t memsize);
-void BufferFree(TBuffer *buf);
-
-
-/*********************************************************************
-** String
-*********************************************************************/
-
-typedef struct
-{
-    TBuffer buffer;
-    uint32_t size;
-} TString;
-
-abyss_bool StringAlloc(TString *s);
-abyss_bool StringConcat(TString *s,char *s2);
-abyss_bool StringBlockConcat(TString *s,char *s2,char **ref);
-void StringFree(TString *s);
-char *StringData(TString *s);
-
-
 /*********************************************************************
 ** Range
 *********************************************************************/
 
+XMLRPC_ABYSS_EXPORTED
 abyss_bool
-RangeDecode(char *str,
-            uint64_t filesize,
-            uint64_t *start,
-            uint64_t *end);
+RangeDecode(char *            const str,
+            xmlrpc_uint64_t   const filesize,
+            xmlrpc_uint64_t * const start,
+            xmlrpc_uint64_t * const end);
 
+XMLRPC_ABYSS_EXPORTED
 abyss_bool DateInit(void);
 
-/*********************************************************************
-** Base64
-*********************************************************************/
-
-void Base64Encode(char *s,char *d);
-
 /*********************************************************************
 ** Session
 *********************************************************************/
 
-abyss_bool SessionLog(TSession *s);
+XMLRPC_ABYSS_EXPORTED
+abyss_bool SessionLog(TSession * const s);
 
 
 #ifdef __cplusplus
